@@ -1,23 +1,30 @@
-swar <-    function(res){
-  data <- attr(res,"data")
-  pdim <- attr(res,"pdim")
-  pmodel <- attr(res,"pmodel")
+swar <- function(y,X,W,id,time,pvar,pdim,pmodel,indexes,cl,...){
+  effect <- pmodel$effect
   twoways <- pmodel$effect=="twoways"
   balanced <- pdim$balanced
-  effect <- pmodel$effect
-  within <- res$within
   T <- pdim$nT$T
   n <- pdim$nT$n
   Kb <- pdim$Kb
-  Kw <- pdim$Kw
   K <- pdim$K
   N <- pdim$nT$N
   Ti <- pdim$Tint$Ti
-  id <- data$id
-  time <- data$time
+  within <- plm.within(y,X,W,id,time,pvar,pdim,pmodel,indexes,cl,...)
+  Kw <- attr(within,"pdim")$Kw
+  if (twoways){
+    pmodel.time <- pmodel.id <- pmodel
+    pmodel.time$effect <- "time"
+    pmodel.id$effect <- "id"
+    between.id <- plm.between(y,X,W,id,time,pvar,pdim,pmodel.id,indexes,cl,...)
+    between.time <- plm.between(y,X,W,id,time,pvar,pdim,pmodel.time,indexes,cl,...)
+    Kb <- attr(between.id,"pdim")$Kb
+  }
+  else{
+    between <- plm.between(y,X,W,id,time,pvar,pdim,pmodel,indexes,cl,...)
+    Kb <- attr(between,"pdim")$Kb
+  }
+
   sigma2 <- list()
   if(!twoways){
-    between <- res$between
     if(balanced){
       ssrbet <- T*sum(between$residuals^2)
       sigma2$one <- ssrbet/(n-Kb-1)
@@ -27,12 +34,17 @@ swar <-    function(res){
       z <- list(sigma2=sigma2,theta=theta)
     }
     else{
-      if (effect=="individual") condvar <- id else condvar <- time
-      X <- data$X$X
-      X.m <- data$X$X.m
-#      X.sum <- attr(psum(cbind(1,X),condvar),"cm")
+      data.name <- within$call$data
+      if (effect=="individual"){
+        condvar <- eval(data.name)[[indexes$id]]
+      }
+      else{
+        condvar <- eval(data.name)[[indexes$time]]
+      }
+      X.m <- papply(cbind(1,X),mymean,condvar)
       X.sum <- attr(papply(cbind(1,X),mysum,condvar),"cm")
-      tr <- sum(diag(solve(crossprod(cbind(1,X.m)))%*%crossprod(X.sum)))
+#      tr <- sum(diag(solve(crossprod(cbind(1,X.m)))%*%crossprod(X.sum))
+      tr <- sum(diag(solve(crossprod(X.m))%*%crossprod(X.sum)))
       sigma2$idios <- sum(within$residuals^2)/(N-n-Kw)
       ssrbet <- sum(between$residuals^2*Ti)
       sigma2$id <- (ssrbet-(n-Kb-1)*sigma2$idios)/(N-tr)
@@ -42,10 +54,8 @@ swar <-    function(res){
   else{
     if(balanced){
       theta=list()
-      between <- res$between
-      between.time <- res$between.time
       sigma2$idios <- sum(within$residuals^2)/((n-1)*(T-1)-Kw)
-      lambda2 <- T*sum(between$residuals^2)/(n-Kb-1)
+      lambda2 <- T*sum(between.id$residuals^2)/(n-Kb-1)
       lambda3 <- n*sum(between.time$residuals^2)/(T-Kb-1)
       lambda4 <- lambda2+lambda3-sigma2$idios
       sigma2$id <- (lambda2-sigma2$idios)/T
@@ -63,20 +73,13 @@ swar <-    function(res){
   z
 }
 
-walhus <- function(res){
-  data <- attr(res,"data")
-  pdim <- attr(res,"pdim")
-  pmodel <- attr(res,"pmodel")
+walhus <- function(y,X,W,id,time,pvar,pdim,pmodel,indexes,cl,...){
   twoways <- pmodel$effect=="twoways"
   effect <- pmodel$effect
   balanced <- pdim$balanced
-  pooling <- res$pooling
-  id <- data$id
-  time <- data$time
+  pooling <- plm.pooling(y,X,W,id,time,pvar,pdim,pmodel,indexes,cl,...)
   T <- pdim$nT$T
   n <- pdim$nT$n
-  Kb <- pdim$Kb
-  Kw <- pdim$Kw
   sigma2=list()
   if(!balanced){stop("walhus not implemented for unbalanced panels\n")}
   if(!twoways){
@@ -90,7 +93,8 @@ walhus <- function(res){
   }
   else{
     theta=list()
-    sigma2$idios <- sum((pooling$residuals-tapply(pooling$residuals,id,mean)[as.character(id)]-tapply(pooling$residuals,time,mean)[as.character(time)])^2)/((n-1)*(T-1))
+    sigma2$idios <- sum((pooling$residuals-tapply(pooling$residuals,id,mean)[as.character(id)]-
+                         tapply(pooling$residuals,time,mean)[as.character(time)])^2)/((n-1)*(T-1))
     lambda2 <- sum(tapply(pooling$residuals,id,mean)^2)*T/(n-1)
     lambda3 <- sum(tapply(pooling$residuals,time,mean)^2)*n/(T-1)
     lambda4 <- lambda2+lambda3-sigma2$idios
@@ -105,20 +109,15 @@ walhus <- function(res){
   z
 }
 
-amemiya <- function(res){
-  data <- attr(res,"data")
-  pdim <- attr(res,"pdim")
-  pmodel <- attr(res,"pmodel")
+amemiya <- function(y,X,W,id,time,pvar,pdim,pmodel,indexes,cl,...){
   twoways <- pmodel$effect=="twoways"
   effect <- pmodel$effect
   balanced <- pdim$balanced
-  within <- res$within
-  id <- data$id
-  time <- data$time
+  within <- plm.within(y,X,W,id,time,pvar,pdim,pmodel,indexes,cl,...)
   T <- pdim$nT$T
   n <- pdim$nT$n
-  Kb <- pdim$Kb
-  Kw <- pdim$Kw
+  Kw <- sum(pvar$time.variation)
+  Kb <- sum(pvar$id.variation)
   sigma2 <- list()
   if(!balanced)stop("amemiya variance decomposition not implemented for unbalanced panels")
   if(!twoways){
@@ -132,10 +131,9 @@ amemiya <- function(res){
   }
   else{
     theta=list()
-    y <- data$y$y
-    X <- data$X$X
     uest <- as.vector(y-within$alpha-X%*%within$coef)
-    sigma2$idios <- sum((uest-tapply(uest,id,mean)[as.character(id)]-tapply(uest,time,mean)[as.character(time)])^2)/((n-1)*(T-1))
+    sigma2$idios <- sum((uest-tapply(uest,id,mean)[as.character(id)]-
+                         tapply(uest,time,mean)[as.character(time)])^2)/((n-1)*(T-1))
     lambda2 <- sum(tapply(uest,id,mean)^2)*T/(n-1)
     lambda3 <- sum(tapply(uest,time,mean)^2)*n/(T-1)
     lambda4 <- lambda2+lambda3-sigma2$idios
@@ -150,26 +148,24 @@ amemiya <- function(res){
   z
 }
 
-nerlove <- function(res){
-  data <- attr(res,"data")
-  pdim <- attr(res,"pdim")
-  pmodel <- attr(res,"pmodel")
+nerlove <- function(y,X,W,id,time,pvar,pdim,pmodel,indexes,cl,...){
+  within <- plm.within(y,X,W,id,time,pvar,pdim,pmodel,indexes,cl,...)
   twoways <- pmodel$effect=="twoways"
   effect <- pmodel$effect
   balanced <- pdim$balanced
-  within <- res$within
   n <- pdim$nT$n
   N <- pdim$nT$N
   Kb <- pdim$Kb
   Kw <- pdim$Kw
   sigma2=list()
   if(!twoways & balanced){
-      sigma2$idios <- sum(within$residuals^2)/N
-      sigma2$id <- sum((within$FE-mean(within$FE))^2)/(n-1)
-      sigma2$one <- T*sigma2$id+sigma2$idios
-      theta <- 1-sqrt(sigma2$idios/sigma2$one)
-      z <- list(theta=theta,sigma2=sigma2)
-    }
-    else stop("nerlove variance decomposition only implemented for balanced oneway panels")
+    sigma2$idios <- sum(within$residuals^2)/N
+    sigma2$id <- sum((within$FE-mean(within$FE))^2)/(n-1)
+    sigma2$one <- T*sigma2$id+sigma2$idios
+    theta <- 1-sqrt(sigma2$idios/sigma2$one)
+    z <- list(theta=theta,sigma2=sigma2)
+  }
+  else stop("nerlove variance decomposition only implemented for balanced oneway panels")
   z
 }
+
